'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.levels = exports.statuses = undefined;

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _asyncToGenerator2 = require('babel-runtime/helpers/asyncToGenerator');

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

var _constants = require('./constants');

Object.defineProperty(exports, 'statuses', {
  enumerable: true,
  get: function get() {
    return _constants.statuses;
  }
});
Object.defineProperty(exports, 'levels', {
  enumerable: true,
  get: function get() {
    return _constants.levels;
  }
});
exports.getAppManagerScript = getAppManagerScript;
exports.appManager = appManager;
exports.default = getAppManagerCallback;

var _appManagerError = require('./utils/app-manager-error');

var _appManagerError2 = _interopRequireDefault(_appManagerError);

var _state = require('./state');

var _state2 = _interopRequireDefault(_state);

var _reconcile = require('./reconcile');

var _reconcile2 = _interopRequireDefault(_reconcile);

var _script = require('./script');

var _script2 = _interopRequireDefault(_script);

var _element = require('./element');

var _element2 = _interopRequireDefault(_element);

var _lifecycle = require('./lifecycle');

var _lifecycle2 = _interopRequireDefault(_lifecycle);

var _update = require('./update');

var _update2 = _interopRequireDefault(_update);

var _fragments = require('./fragments');

var _fragments2 = _interopRequireDefault(_fragments);

var _context6 = require('./utils/context');

var _context7 = _interopRequireDefault(_context6);

var _wrapScript = require('./utils/wrap-script');

var _wrapScript2 = _interopRequireDefault(_wrapScript);

var _config = require('./utils/config');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function getAppManagerScript(config, events, options) {
  var slots = (0, _config.setNames)(config.slots);
  var fragments = (0, _config.setNames)(config.fragments);
  var routes = (0, _config.setNames)(config.routes);

  var importTimeout = _constants.IMPORT_TIMEOUT;
  var getRouteName = (0, _config.defaultGetRouteName)(routes);
  var getElement = _config.defaultGetElement;
  var onRecoverableError = function onRecoverableError(err) {
    throw err;
  };
  var getAdditionalState = void 0;

  if (options) {
    if (typeof options.importTimeout === 'number') {
      importTimeout = options.importTimeout;
    }

    if (typeof options.getRouteName === 'function') {
      getRouteName = options.getRouteName;
    }

    if (typeof options.getElement === 'function') {
      getElement = options.getElement;
    }

    if (typeof options.getAdditionalState === 'function') {
      getAdditionalState = options.getAdditionalState;
    }
  }

  if (events && typeof events.emit === 'function') {
    onRecoverableError = events.emit.bind(events, _constants.eventTitles.WARNING);
  }

  var updateState = (0, _state2.default)(routes, getRouteName, getAdditionalState);
  var getPageFragments = (0, _fragments2.default)(slots, fragments, routes);
  var reconcile = (0, _reconcile2.default)(getPageFragments);
  var getSlotElement = (0, _element2.default)(slots, getElement, importTimeout);

  var context = new _context7.default();

  var getFragmentScript = (0, _script2.default)(fragments, importTimeout, context);
  var createLifecycle = (0, _lifecycle2.default)(slots, getFragmentScript, getSlotElement, context, onRecoverableError);
  var updatePage = (0, _update2.default)(reconcile, createLifecycle, onRecoverableError);

  var script = {
    version: 6,

    hydrate: function () {
      var _ref = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee(container, parentState) {
        var state;
        return _regenerator2.default.wrap(function _callee$(_context) {
          while (1) {
            switch (_context.prev = _context.next) {
              case 0:
                _context.next = 2;
                return updateState(parentState);

              case 2:
                state = _context.sent;

                if (state.route) {
                  _context.next = 5;
                  break;
                }

                throw new _appManagerError2.default('No route found on hydrate', {
                  source: 'hydrate_script',
                  recoverable: false,
                  level: _constants.levels.ERROR,
                  fragment: null,
                  slot: null,
                  code: 'no_route'
                });

              case 5:

                context.setState(state);

                _context.next = 8;
                return updatePage(container, state);

              case 8:
              case 'end':
                return _context.stop();
            }
          }
        }, _callee, this);
      }));

      function hydrate(_x, _x2) {
        return _ref.apply(this, arguments);
      }

      return hydrate;
    }(),
    render: function () {
      var _ref2 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee2(container, parentState) {
        var state;
        return _regenerator2.default.wrap(function _callee2$(_context2) {
          while (1) {
            switch (_context2.prev = _context2.next) {
              case 0:
                _context2.next = 2;
                return updateState(parentState);

              case 2:
                state = _context2.sent;

                if (state.route) {
                  _context2.next = 5;
                  break;
                }

                throw new _appManagerError2.default('No route found on render', {
                  source: 'render_script',
                  recoverable: false,
                  level: _constants.levels.ERROR,
                  fragment: null,
                  slot: null,
                  code: 'no_route'
                });

              case 5:

                context.setState(state);

                if (!(options && typeof options.getLayout === 'function')) {
                  _context2.next = 10;
                  break;
                }

                _context2.next = 9;
                return options.getLayout(state);

              case 9:
                container.innerHTML = _context2.sent;

              case 10:
                _context2.next = 12;
                return updatePage(container, state);

              case 12:
              case 'end':
                return _context2.stop();
            }
          }
        }, _callee2, this);
      }));

      function render(_x3, _x4) {
        return _ref2.apply(this, arguments);
      }

      return render;
    }(),
    onStateChange: function () {
      var _ref3 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee3(container, parentState) {
        var state;
        return _regenerator2.default.wrap(function _callee3$(_context3) {
          while (1) {
            switch (_context3.prev = _context3.next) {
              case 0:
                _context3.next = 2;
                return updateState(parentState, context.state.route);

              case 2:
                state = _context3.sent;

                if (state.route) {
                  _context3.next = 5;
                  break;
                }

                throw new _appManagerError2.default('No route found on state change', {
                  source: 'on_state_change_script',
                  recoverable: false,
                  level: _constants.levels.ERROR,
                  fragment: null,
                  slot: null,
                  code: 'no_route'
                });

              case 5:

                context.setState(state);

                _context3.next = 8;
                return updatePage(container, state);

              case 8:
              case 'end':
                return _context3.stop();
            }
          }
        }, _callee3, this);
      }));

      function onStateChange(_x5, _x6) {
        return _ref3.apply(this, arguments);
      }

      return onStateChange;
    }(),
    unmount: function () {
      var _ref4 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee4(container, parentState) {
        var state;
        return _regenerator2.default.wrap(function _callee4$(_context4) {
          while (1) {
            switch (_context4.prev = _context4.next) {
              case 0:
                _context4.next = 2;
                return updateState(parentState, context.state.route);

              case 2:
                state = _context4.sent;


                context.setState(state);

                _context4.next = 6;
                return updatePage(container, state);

              case 6:
              case 'end':
                return _context4.stop();
            }
          }
        }, _callee4, this);
      }));

      function unmount(_x7, _x8) {
        return _ref4.apply(this, arguments);
      }

      return unmount;
    }()
  };

  // $FlowFixMe until https://github.com/facebook/flow/issues/285 is fixed.
  return Object.defineProperty(script, 'state', {
    enumerable: true,
    get: function get() {
      return context.state;
    }
  });
}

function appManager(config, events, options) {
  var appManagerScript = getAppManagerScript(config, events, options);
  return (0, _wrapScript2.default)(appManagerScript);
}

function getAppManagerCallback(config, events, options) {
  var appManagerScript = getAppManagerScript(config, events, options);

  return function () {
    var _ref5 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee5(container, state) {
      return _regenerator2.default.wrap(function _callee5$(_context5) {
        while (1) {
          switch (_context5.prev = _context5.next) {
            case 0:
              if (!(state.eventTitle === _constants.eventTitles.INITIALISE)) {
                _context5.next = 5;
                break;
              }

              _context5.next = 3;
              return appManagerScript.hydrate(container, state);

            case 3:
              _context5.next = 12;
              break;

            case 5:
              if (!(state.eventTitle === _constants.eventTitles.WINDOW_BEFORE_UNLOAD)) {
                _context5.next = 10;
                break;
              }

              _context5.next = 8;
              return appManagerScript.unmount(container, state);

            case 8:
              _context5.next = 12;
              break;

            case 10:
              _context5.next = 12;
              return appManagerScript.onStateChange(container, state);

            case 12:
            case 'end':
              return _context5.stop();
          }
        }
      }, _callee5, this);
    }));

    function appManagerCallback(_x9, _x10) {
      return _ref5.apply(this, arguments);
    }

    return appManagerCallback;
  }();
}