'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _asyncToGenerator2 = require('babel-runtime/helpers/asyncToGenerator');

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

exports.default = iniHistoryCallback;

var _constants = require('./constants');

var _config = require('./utils/config');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function iniHistoryCallback(w) {
  if (!w.document || !w.location || !w.history) {
    throw new Error('Invalid window object. Aborting.');
  }

  function getCurrentResource() {
    var _w$location = w.location,
        pathname = _w$location.pathname,
        search = _w$location.search;

    return '' + pathname + (search || '');
  }

  function getHistoryState() {
    return w.history.state;
  }

  function getCurrentTitle() {
    return w.document.title;
  }

  function shimHistory(events, callback) {
    /* eslint-disable no-param-reassign */

    var onpopstate = w.onpopstate,
        onbeforeunload = w.onbeforeunload,
        history = w.history;
    var pushState = history.pushState,
        replaceState = history.replaceState;


    w.history.pushState = function hcPushState() {
      if (typeof pushState === 'function') {
        for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
          args[_key] = arguments[_key];
        }

        pushState.apply(w.history, args);
      }

      events.emit(_constants.eventTitles.HISTORY_PUSH_STATE);
      events.emit(_constants.eventTitles.HISTORY_STATE_CHANGE, _constants.eventTitles.HISTORY_PUSH_STATE);
    };

    w.history.replaceState = function hcReplaceState() {
      if (typeof replaceState === 'function') {
        for (var _len2 = arguments.length, args = Array(_len2), _key2 = 0; _key2 < _len2; _key2++) {
          args[_key2] = arguments[_key2];
        }

        replaceState.apply(w.history, args);
      }

      events.emit(_constants.eventTitles.HISTORY_REPLACE_STATE);
      events.emit(_constants.eventTitles.HISTORY_STATE_CHANGE, _constants.eventTitles.HISTORY_REPLACE_STATE);
    };

    w.onpopstate = function hcOnpopstate() {
      if (typeof onpopstate === 'function') {
        for (var _len3 = arguments.length, args = Array(_len3), _key3 = 0; _key3 < _len3; _key3++) {
          args[_key3] = arguments[_key3];
        }

        onpopstate.apply(w.history, args);
      }

      events.emit(_constants.eventTitles.HISTORY_POP_STATE);
      events.emit(_constants.eventTitles.HISTORY_STATE_CHANGE, _constants.eventTitles.HISTORY_POP_STATE);
    };

    w.onbeforeunload = function hcOnbeforeunload() {
      if (typeof onbeforeunload === 'function') {
        for (var _len4 = arguments.length, args = Array(_len4), _key4 = 0; _key4 < _len4; _key4++) {
          args[_key4] = arguments[_key4];
        }

        onbeforeunload.apply(w.history, args);
      }

      events.emit(_constants.eventTitles.WINDOW_BEFORE_UNLOAD);
      events.removeListener(_constants.eventTitles.HISTORY_STATE_CHANGE, callback);

      w.onpopstate = onpopstate;
      w.history.pushState = pushState;
      w.history.replaceState = replaceState;
    };

    events.on(_constants.eventTitles.HISTORY_STATE_CHANGE, callback);
  }

  return function historyCallback(callback, events, options) {
    var stateChanger = function () {
      var _ref = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee(eventTitle) {
        var parentElement, getElement, rootElement, nextStateChange;
        return _regenerator2.default.wrap(function _callee$(_context) {
          while (1) {
            switch (_context.prev = _context.next) {
              case 0:
                _context.prev = 0;
                parentElement = void 0;
                getElement = _config.defaultGetElement;

                if (!options) {
                  _context.next = 9;
                  break;
                }

                if (typeof options.getElement === 'function') {
                  getElement = options.getElement;
                }

                if (!(typeof options.parentQuerySelector === 'string')) {
                  _context.next = 9;
                  break;
                }

                _context.next = 8;
                return getElement(w.document, options.parentQuerySelector);

              case 8:
                parentElement = _context.sent;

              case 9:
                rootElement = parentElement || w.document.body;
                _context.next = 12;
                return callback(rootElement, {
                  rootState: true,
                  resource: getCurrentResource(),
                  title: getCurrentTitle(),
                  historyState: getHistoryState(),
                  eventTitle: eventTitle
                });

              case 12:
                nextStateChange = queuedStateChange;

                if (!(typeof nextStateChange !== 'string')) {
                  _context.next = 16;
                  break;
                }

                runningStateChange = null;
                return _context.abrupt('return');

              case 16:

                queuedStateChange = null;
                runningStateChange = stateChanger(nextStateChange);
                _context.next = 24;
                break;

              case 20:
                _context.prev = 20;
                _context.t0 = _context['catch'](0);

                runningStateChange = null;
                events.emit(_constants.eventTitles.ERROR, _context.t0);

              case 24:
              case 'end':
                return _context.stop();
            }
          }
        }, _callee, this, [[0, 20]]);
      }));

      return function stateChanger(_x) {
        return _ref.apply(this, arguments);
      };
    }();

    if (!events || typeof events.emit !== 'function' || typeof events.on !== 'function' || typeof events.removeListener !== 'function') {
      throw new Error('Invalid events object. Aborting.');
    }

    var runningStateChange = null;
    var queuedStateChange = null;

    function handleStateChange(eventTitle) {
      if (runningStateChange) {
        queuedStateChange = eventTitle;
      } else {
        runningStateChange = stateChanger(eventTitle);
      }
    }

    shimHistory(events, handleStateChange);

    handleStateChange(_constants.eventTitles.INITIALISE);

    return {
      get runningStateChange() {
        return runningStateChange;
      },

      get queuedStateChange() {
        return queuedStateChange;
      }
    };
  };
}