'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _promise = require('babel-runtime/core-js/promise');

var _promise2 = _interopRequireDefault(_promise);

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _defineProperty2 = require('babel-runtime/helpers/defineProperty');

var _defineProperty3 = _interopRequireDefault(_defineProperty2);

var _toConsumableArray2 = require('babel-runtime/helpers/toConsumableArray');

var _toConsumableArray3 = _interopRequireDefault(_toConsumableArray2);

var _keys = require('babel-runtime/core-js/object/keys');

var _keys2 = _interopRequireDefault(_keys);

var _extends2 = require('babel-runtime/helpers/extends');

var _extends3 = _interopRequireDefault(_extends2);

var _asyncToGenerator2 = require('babel-runtime/helpers/asyncToGenerator');

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

exports.default = appManager;

var _appManagerError = require('./utils/app-manager-error');

var _appManagerError2 = _interopRequireDefault(_appManagerError);

var _config = require('./utils/config');

var _constants = require('./constants');

var _reconcile2 = require('./reconcile');

var _reconcile3 = _interopRequireDefault(_reconcile2);

var _state = require('./state');

var _state2 = _interopRequireDefault(_state);

var _fragments = require('./fragments');

var _fragments2 = _interopRequireDefault(_fragments);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function appManager(config) {
  var options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

  var slots = (0, _config.setNames)(config.slots);
  var routes = (0, _config.setNames)(config.routes);
  var fragments = (0, _config.setNames)(config.fragments);
  var getPageFragments = (0, _fragments2.default)(slots, fragments, routes);
  var reconcile = (0, _reconcile3.default)(getPageFragments);

  var getRouteName = options.getRouteName || (0, _config.defaultGetRouteName)(routes);
  var updateState = (0, _state2.default)(routes, getRouteName, options.getAdditionalState);

  function getState(resource) {
    return updateState({
      serverState: true,
      eventTitle: _constants.eventTitles.INITIALISE,
      resource: resource
    });
  }

  return {
    getState: getState,

    getSlotsMarkup: function () {
      var _ref = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee2(resource) {
        var _this = this;

        for (var _len = arguments.length, ssrGetMarkupArgs = Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
          ssrGetMarkupArgs[_key - 1] = arguments[_key];
        }

        var errData, state, newRouteName, _reconcile, mount, markupPromises, resolvedMarkup;

        return _regenerator2.default.wrap(function _callee2$(_context2) {
          while (1) {
            switch (_context2.prev = _context2.next) {
              case 0:
                errData = {
                  source: 'get_slots_markup',
                  fragment: null,
                  slot: null
                };
                _context2.next = 3;
                return getState(resource);

              case 3:
                state = _context2.sent;

                if (!(!state || !state.route)) {
                  _context2.next = 6;
                  break;
                }

                throw new _appManagerError2.default('Could not find a route to match path', (0, _extends3.default)({}, errData, {
                  level: _constants.levels.ERROR,
                  code: 'invalid_route',
                  recoverable: false
                }));

              case 6:
                newRouteName = state.route.name;
                _reconcile = reconcile(newRouteName), mount = _reconcile.mount;
                markupPromises = (0, _keys2.default)(mount).map(function () {
                  var _ref2 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee(slotName) {
                    var slot, markup, fragmentName, fragment;
                    return _regenerator2.default.wrap(function _callee$(_context) {
                      while (1) {
                        switch (_context.prev = _context.next) {
                          case 0:
                            slot = slots[slotName];
                            markup = '';
                            _context.prev = 2;
                            fragmentName = mount[slotName];

                            if (!fragmentName) {
                              _context.next = 10;
                              break;
                            }

                            fragment = fragments[fragmentName];

                            if (!(typeof fragment.ssrGetMarkup === 'function')) {
                              _context.next = 10;
                              break;
                            }

                            _context.next = 9;
                            return fragment.ssrGetMarkup.apply(fragment, [slot.querySelector, state].concat((0, _toConsumableArray3.default)(ssrGetMarkupArgs)));

                          case 9:
                            markup = _context.sent;

                          case 10:
                            _context.next = 20;
                            break;

                          case 12:
                            _context.prev = 12;
                            _context.t0 = _context['catch'](2);

                            if (!(options.ssrHaltOnError === true)) {
                              _context.next = 16;
                              break;
                            }

                            throw new _appManagerError2.default(_context.t0, (0, _extends3.default)({}, errData, {
                              level: _constants.levels.ERROR,
                              code: 'get_slot_markup',
                              recoverable: false,
                              slot: slotName
                            }));

                          case 16:
                            if (!(typeof slot.ssrGetErrorMarkup === 'function')) {
                              _context.next = 20;
                              break;
                            }

                            _context.next = 19;
                            return slot.ssrGetErrorMarkup.apply(slot, [slot.querySelector, state].concat((0, _toConsumableArray3.default)(ssrGetMarkupArgs)));

                          case 19:
                            markup = _context.sent;

                          case 20:
                            return _context.abrupt('return', (0, _defineProperty3.default)({}, slotName, markup));

                          case 21:
                          case 'end':
                            return _context.stop();
                        }
                      }
                    }, _callee, _this, [[2, 12]]);
                  }));

                  return function (_x3) {
                    return _ref2.apply(this, arguments);
                  };
                }());
                resolvedMarkup = void 0;
                _context2.prev = 10;
                _context2.next = 13;
                return _promise2.default.all(markupPromises);

              case 13:
                resolvedMarkup = _context2.sent;
                _context2.next = 19;
                break;

              case 16:
                _context2.prev = 16;
                _context2.t0 = _context2['catch'](10);
                throw new _appManagerError2.default(_context2.t0, (0, _extends3.default)({}, errData, {
                  level: _constants.levels.ERROR,
                  code: 'get_markup',
                  recoverable: false
                }));

              case 19:
                return _context2.abrupt('return', resolvedMarkup.reduce(function (acc, slotMarkup) {
                  return (0, _extends3.default)({}, acc, slotMarkup);
                }, {}));

              case 20:
              case 'end':
                return _context2.stop();
            }
          }
        }, _callee2, this, [[10, 16]]);
      }));

      function getSlotsMarkup(_x2) {
        return _ref.apply(this, arguments);
      }

      return getSlotsMarkup;
    }()
  };
}