"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _defineProperty2 = require("babel-runtime/helpers/defineProperty");

var _defineProperty3 = _interopRequireDefault(_defineProperty2);

var _keys = require("babel-runtime/core-js/object/keys");

var _keys2 = _interopRequireDefault(_keys);

var _toConsumableArray2 = require("babel-runtime/helpers/toConsumableArray");

var _toConsumableArray3 = _interopRequireDefault(_toConsumableArray2);

var _extends3 = require("babel-runtime/helpers/extends");

var _extends4 = _interopRequireDefault(_extends3);

exports.default = findSlots;

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function formatThings(slots, things) {
  return things.filter(function (thing) {
    return thing && Array.isArray(thing.slots);
  }).map(function (thing) {
    return (0, _extends4.default)({}, thing, { slots: thing.slots.filter(function (slotName) {
        return Object.prototype.hasOwnProperty.call(slots, slotName);
      }) });
  }).filter(function (thing) {
    return thing.slots.length > 0;
  });
}

function filterLessImportantThingSlots(things, neededSlot, startIndex) {
  function filterThingSlots(filteredThings, thing, index) {
    if (index <= startIndex) {
      return filteredThings;
    }

    return [].concat((0, _toConsumableArray3.default)(filteredThings.slice(0, index)), [(0, _extends4.default)({}, thing, { slots: thing.slots.filter(function (slotName) {
        return slotName !== neededSlot;
      }) })], (0, _toConsumableArray3.default)(filteredThings.slice(index + 1)));
  }

  return things.reduce(filterThingSlots, things);
}

function ensurePriorityThings(things) {
  function filterThingSlots(filteredThings, thing, index) {
    return thing.slots.length === 1 ? filterLessImportantThingSlots(filteredThings, thing.slots[0], index) : filteredThings;
  }

  return things.reduce(filterThingSlots, things);
}

function getChain(things) {
  var usedSlots = [];
  var index = 0;

  while (index < things.length) {
    var _slot = things[index].slots[0];

    if (usedSlots.indexOf(_slot) !== -1) {
      return things.slice(0, index);
    }

    usedSlots.push(_slot);
    index += 1;
  }

  return things;
}

function calculateSlots(allThings, numSlots) {
  function getSlots(test, offset, things, longestChain) {
    function findLongestChainInThings(thingChain, thing, thingIndex) {
      if (thingChain.length >= numSlots) {
        return thingChain;
      }

      var lessImportantThings = things.slice(thingIndex + 1);

      function findLongestChainInSlots(slotChain, slot) {
        if (slotChain.length >= numSlots) {
          return slotChain;
        }

        var nextTest = [].concat((0, _toConsumableArray3.default)(test.slice(0, thingIndex + offset)), [(0, _extends4.default)({}, thing, { slots: [slot] })], (0, _toConsumableArray3.default)(test.slice(thingIndex + offset + 1)));

        if (lessImportantThings.length > 0) {
          return getSlots(nextTest, offset + 1, lessImportantThings, slotChain);
        }

        var nextChain = getChain(nextTest);
        return nextChain.length > slotChain.length ? nextChain : slotChain;
      }

      return thing.slots.reduce(findLongestChainInSlots, thingChain);
    }

    return things.reduce(findLongestChainInThings, longestChain);
  }

  var seed = allThings.map(function (thing) {
    return (0, _extends4.default)({}, thing, { slots: [thing.slots[0]] });
  });
  return getSlots(seed, 0, allThings, []);
}

function findSlots() {
  var slots = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
  var things = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : [];

  var numSlots = (0, _keys2.default)(slots).length;
  var managedThings = formatThings(slots, things);
  var ensuredThings = [].concat((0, _toConsumableArray3.default)(Array(numSlots))).reduce(ensurePriorityThings, managedThings);
  var filteredThings = calculateSlots(ensuredThings, numSlots);
  var thingsObject = filteredThings.reduce(function (thingsInSlots, thing) {
    return (0, _extends4.default)({}, thingsInSlots, (0, _defineProperty3.default)({}, thing.slots[0], thing.name));
  }, {});
  return (0, _extends4.default)({}, slots, thingsObject);
}